@@ -19,8 +19,10 @@ package cmd
 
 import (
 	"context"
+	"fmt"
 	"io/ioutil"
 	"os"
+	"path/filepath"
 	"testing"
 
 	v1 "github.com/apache/camel-k/pkg/apis/camel/v1"
@@ -46,6 +48,17 @@ func initializeRunCmdOptions(t *testing.T) (*runCmdOptions, *cobra.Command, Root
 	return runCmdOptions, rootCmd, *options
 }
 
+// nolint: unparam
+func initializeRunCmdOptionsWithOutput(t *testing.T) (*runCmdOptions, *cobra.Command, RootCmdOptions) {
+	t.Helper()
+
+	options, rootCmd := kamelTestPreAddCommandInit()
+	runCmdOptions := addTestRunCmdWithOutput(*options, rootCmd)
+	kamelTestPostAddCommandInit(t, rootCmd)
+
+	return runCmdOptions, rootCmd, *options
+}
+
 func addTestRunCmd(options RootCmdOptions, rootCmd *cobra.Command) *runCmdOptions {
 	// add a testing version of run Command
 	runCmd, runOptions := newCmdRun(&options)
@@ -60,6 +73,17 @@ func addTestRunCmd(options RootCmdOptions, rootCmd *cobra.Command) *runCmdOption
 	return runOptions
 }
 
+func addTestRunCmdWithOutput(options RootCmdOptions, rootCmd *cobra.Command) *runCmdOptions {
+	// add a testing version of run Command with output
+	runCmd, runOptions := newCmdRun(&options)
+	runCmd.PersistentPreRunE = func(c *cobra.Command, args []string) error {
+		return nil
+	}
+	runCmd.Args = test.ArbitraryArgs
+	rootCmd.AddCommand(runCmd)
+	return runOptions
+}
+
 func TestRunNoFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun, integrationSource)
@@ -87,18 +111,6 @@ func TestRunCompressionFlag(t *testing.T) {
 	assert.Equal(t, true, runCmdOptions.Compression)
 }
 
-func TestRunConfigMapFlag(t *testing.T) {
-	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
-	_, err := test.ExecuteCommand(rootCmd, cmdRun,
-		"--configmap", "configMap1",
-		"--configmap", "configMap2",
-		integrationSource)
-	assert.Nil(t, err)
-	assert.Len(t, runCmdOptions.ConfigMaps, 2)
-	assert.Equal(t, "configMap1", runCmdOptions.ConfigMaps[0])
-	assert.Equal(t, "configMap2", runCmdOptions.ConfigMaps[1])
-}
-
 func TestRunDependencyFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun,
@@ -120,6 +132,16 @@ func TestRunDevFlag(t *testing.T) {
 	assert.Equal(t, true, runCmdOptions.Dev)
 }
 
+func TestRunDevModeOutputFlag(t *testing.T) {
+	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
+	_, err := test.ExecuteCommand(rootCmd, cmdRun, "--dev", "-o", "yaml", integrationSource)
+	assert.Equal(t, true, runCmdOptions.Dev)
+	assert.Equal(t, "yaml", runCmdOptions.OutputFormat)
+	assert.NotNil(t, err)
+	assert.Equal(t, "cannot use --dev with -o/--output option",
+		err.Error())
+}
+
 func TestRunEnvFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun,
@@ -190,13 +212,22 @@ func TestRunNameFlag(t *testing.T) {
 func TestRunOpenApiFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun,
-		"--open-api", "oapi1",
-		"--open-api", "oapi2",
+		"--open-api", "file:oapi1",
+		"--open-api", "configmap:oapi2",
 		integrationSource)
 	assert.Nil(t, err)
 	assert.Len(t, runCmdOptions.OpenAPIs, 2)
-	assert.Equal(t, "oapi1", runCmdOptions.OpenAPIs[0])
-	assert.Equal(t, "oapi2", runCmdOptions.OpenAPIs[1])
+	assert.Equal(t, "file:oapi1", runCmdOptions.OpenAPIs[0])
+	assert.Equal(t, "configmap:oapi2", runCmdOptions.OpenAPIs[1])
+}
+
+func TestRunOpenApiInvalidFlag(t *testing.T) {
+	_, rootCmd, _ := initializeRunCmdOptions(t)
+	_, err := test.ExecuteCommand(rootCmd, cmdRun,
+		"--open-api", "secret:oapi1",
+		"--open-api", "oapi2",
+		integrationSource)
+	assert.NotNil(t, err)
 }
 
 func TestRunOutputFlag(t *testing.T) {
@@ -227,15 +258,6 @@ func TestRunPropertyFlag(t *testing.T) {
 	assert.Equal(t, "property3", runCmdOptions.Properties[2])
 }
 
-func TestRunPropertyFileFlagMissingFileExtension(t *testing.T) {
-	_, rootCmd, _ := initializeRunCmdOptions(t)
-	_, err := test.ExecuteCommand(rootCmd, cmdRun,
-		"--property-file", "file1",
-		"--property-file", "file2",
-		integrationSource)
-	assert.NotNil(t, err)
-}
-
 const TestPropertyFileContent = `
 a=b
 # There's an issue in the properties lib: https://github.com/magiconair/properties/issues/59
@@ -255,7 +277,7 @@ func TestAddPropertyFile(t *testing.T) {
 	}
 
 	assert.Nil(t, tmpFile.Close())
-	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestPropertyFileContent), 0o644))
+	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestPropertyFileContent), 0o400))
 
 	properties, err := convertToTraitParameter("file:"+tmpFile.Name(), "trait.properties")
 	assert.Nil(t, err)
@@ -265,25 +287,6 @@ func TestAddPropertyFile(t *testing.T) {
 	assert.Equal(t, `trait.properties=i = j\nk`, properties[2])
 }
 
-func TestRunPropertyFileFlag(t *testing.T) {
-	var tmpFile *os.File
-	var err error
-	if tmpFile, err = ioutil.TempFile("", "camel-k-*.properties"); err != nil {
-		t.Error(err)
-	}
-
-	assert.Nil(t, tmpFile.Close())
-	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestPropertyFileContent), 0o644))
-
-	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
-	_, errExecute := test.ExecuteCommand(rootCmd, cmdRun,
-		"--property-file", tmpFile.Name(),
-		integrationSource)
-	assert.Nil(t, errExecute)
-	assert.Len(t, runCmdOptions.PropertyFiles, 1)
-	assert.Equal(t, tmpFile.Name(), runCmdOptions.PropertyFiles[0])
-}
-
 func TestRunProperty(t *testing.T) {
 	properties, err := convertToTraitParameter(`key=value\nnewline`, "trait.properties")
 	assert.Nil(t, err)
@@ -310,18 +313,6 @@ func TestRunSaveFlag(t *testing.T) {
 	assert.Equal(t, true, runCmdOptions.Save)
 }
 
-func TestRunSecretFlag(t *testing.T) {
-	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
-	_, err := test.ExecuteCommand(rootCmd, cmdRun,
-		"--secret", "secret1",
-		"--secret", "secret2",
-		integrationSource)
-	assert.Nil(t, err)
-	assert.Len(t, runCmdOptions.Secrets, 2)
-	assert.Equal(t, "secret1", runCmdOptions.Secrets[0])
-	assert.Equal(t, "secret2", runCmdOptions.Secrets[1])
-}
-
 func TestRunSourceFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun, "--source", "source1", integrationSource)
@@ -337,16 +328,27 @@ func TestRunSyncFlag(t *testing.T) {
 	assert.Equal(t, true, runCmdOptions.Sync)
 }
 
-func TestRunTraitFlag(t *testing.T) {
+func TestRunExistingTraitFlag(t *testing.T) {
 	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
 	_, err := test.ExecuteCommand(rootCmd, cmdRun,
-		"--trait", "trait1",
-		"--trait", "trait2",
+		"--trait", "jvm.enabled",
+		"--trait", "logging.enabled",
 		integrationSource)
 	assert.Nil(t, err)
 	assert.Len(t, runCmdOptions.Traits, 2)
-	assert.Equal(t, "trait1", runCmdOptions.Traits[0])
-	assert.Equal(t, "trait2", runCmdOptions.Traits[1])
+	assert.Equal(t, "jvm.enabled", runCmdOptions.Traits[0])
+	assert.Equal(t, "logging.enabled", runCmdOptions.Traits[1])
+}
+
+func TestRunMissingTraitFlag(t *testing.T) {
+	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
+	_, err := test.ExecuteCommand(rootCmd, cmdRun,
+		"--trait", "bogus.missing",
+		integrationSource)
+	assert.NotNil(t, err)
+	assert.Equal(t, "bogus.missing is not a valid trait property", err.Error())
+	assert.Len(t, runCmdOptions.Traits, 1)
+	assert.Equal(t, "bogus.missing", runCmdOptions.Traits[0])
 }
 
 func TestConfigureTraits(t *testing.T) {
@@ -378,6 +380,55 @@ func TestConfigureTraits(t *testing.T) {
 	assertTraitConfiguration(t, traits, "prometheus", `{"podMonitor":false}`)
 }
 
+type customTrait struct {
+	trait.BaseTrait `property:",squash"`
+	// SimpleMap
+	SimpleMap  map[string]string            `property:"simple-map" json:"simpleMap,omitempty"`
+	DoubleMap  map[string]map[string]string `property:"double-map" json:"doubleMap,omitempty"`
+	SliceOfMap []map[string]string          `property:"slice-of-map" json:"sliceOfMap,omitempty"`
+}
+
+func (c customTrait) Configure(environment *trait.Environment) (bool, error) {
+	panic("implement me")
+}
+func (c customTrait) Apply(environment *trait.Environment) error {
+	panic("implement me")
+}
+
+var _ trait.Trait = &customTrait{}
+
+type customTraitFinder struct {
+}
+
+func (finder customTraitFinder) GetTrait(id string) trait.Trait {
+	if id == "custom" {
+		return &customTrait{}
+	}
+	return nil
+}
+
+func TestTraitsNestedConfig(t *testing.T) {
+	runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
+	_, err := test.ExecuteCommand(rootCmd, "run",
+		"--trait", "custom.simple-map.a=b",
+		"--trait", "custom.simple-map.y=z",
+		"--trait", "custom.double-map.m.n=q",
+		"--trait", "custom.double-map.m.o=w",
+		"--trait", "custom.slice-of-map[0].f=g",
+		"--trait", "custom.slice-of-map[3].f=h",
+		"--trait", "custom.slice-of-map[2].f=i",
+		"example.js")
+	// We will have an error because those traits are not existing
+	// however we want to test how those properties are mapped in the configuration
+	assert.NotNil(t, err)
+	catalog := &customTraitFinder{}
+	traits, err := configureTraits(runCmdOptions.Traits, catalog)
+
+	assert.Nil(t, err)
+	assert.Len(t, traits, 1)
+	assertTraitConfiguration(t, traits, "custom", `{"simpleMap":{"a":"b","y":"z"},"doubleMap":{"m":{"n":"q","o":"w"}},"sliceOfMap":[{"f":"g"},null,{"f":"i"},{"f":"h"}]}`)
+}
+
 func assertTraitConfiguration(t *testing.T, traits map[string]v1.TraitSpec, trait string, expected string) {
 	t.Helper()
 
@@ -442,51 +493,6 @@ func TestRunValidateArgs(t *testing.T) {
 	assert.Equal(t, "One of the provided sources is not reachable: missing file or unsupported scheme in missing_file", err.Error())
 }
 
-//
-// This test does work when running as single test but fails
-// otherwise as we are using a global viper instance
-//
-
-/*
-const TestKamelConfigContent = `
-kamel:
-  install:
-    olm: false
-  run:
-    integration:
-      route:
-        sources:
-        - examples/dns.js
-        - examples/Sample.java
-`
-
-func TestRunWithSavedValues(t *testing.T) {
-	dir, err := ioutil.TempDir("", "run-")
-	assert.Nil(t, err)
-
-	defer func() {
-		_ = os.RemoveAll(dir)
-	}()
-
-	assert.Nil(t, os.Setenv("KAMEL_CONFIG_PATH", dir))
-	defer func() {
-		_ = os.Unsetenv("KAMEL_CONFIG_PATH")
-	}()
-
-	assert.Nil(t, ioutil.WriteFile(path.Join(dir, "kamel-config.yaml"), []byte(TestKamelConfigContent), 0644))
-
-	options, rootCmd := kamelTestPreAddCommandInit()
-
-	runCmdOptions := addTestRunCmd(options, rootCmd)
-
-	kamelTestPostAddCommandInit(t, rootCmd)
-
-	_, err = test.ExecuteCommand(rootCmd, "run", "route.java")
-
-	assert.Nil(t, err)
-	assert.Len(t, runCmdOptions.Sources, 2)
-}*/
-
 func TestRunBinaryResource(t *testing.T) {
 	binaryResourceSpec, err := binaryOrTextResource("file.ext", []byte{1, 2, 3, 4}, "application/octet-stream", false, v1.ResourceTypeData, "")
 	assert.Nil(t, err)
@@ -532,7 +538,7 @@ func TestRunTextCompressedResource(t *testing.T) {
 }
 
 func TestResolvePodTemplate(t *testing.T) {
-	// runCmdOptions, rootCmd, _ := initializeRunCmdOptions(t)
+	_, rootCmd, _ := initializeRunCmdOptions(t)
 	templateText := `
 containers:
   - name: integration
@@ -548,7 +554,7 @@ volumes:
 `
 
 	integrationSpec := v1.IntegrationSpec{}
-	err := resolvePodTemplate(context.TODO(), templateText, &integrationSpec)
+	err := resolvePodTemplate(context.TODO(), rootCmd, templateText, &integrationSpec)
 	assert.Nil(t, err)
 	assert.NotNil(t, integrationSpec.PodTemplate)
 	assert.Equal(t, 1, len(integrationSpec.PodTemplate.Spec.Containers))
@@ -556,10 +562,11 @@ volumes:
 }
 
 func TestResolveJsonPodTemplate(t *testing.T) {
+	_, rootCmd, _ := initializeRunCmdOptions(t)
 	integrationSpec := v1.IntegrationSpec{}
 	minifiedYamlTemplate := `{"containers": [{"name": "second"}, {"name": "integration", "env": [{"name": "CAMEL_K_DIGEST", "value": "new_value"}]}]}`
 
-	err := resolvePodTemplate(context.TODO(), minifiedYamlTemplate, &integrationSpec)
+	err := resolvePodTemplate(context.TODO(), rootCmd, minifiedYamlTemplate, &integrationSpec)
 
 	assert.Nil(t, err)
 	assert.NotNil(t, integrationSpec.PodTemplate)
@@ -573,3 +580,103 @@ func TestFilterBuildPropertyFiles(t *testing.T) {
 	assert.Equal(t, len(outputValues), 1)
 	assert.Equal(t, outputValues[0], "/tmp/test")
 }
+
+const TestSrcContent = `
+import org.apache.camel.builder.RouteBuilder;
+
+public class Sample extends RouteBuilder {
+  @Override
+  public void configure() throws Exception {
+	  from("timer:tick")
+        .log("Hello Camel K!");
+  }
+}
+`
+
+func TestOutputYaml(t *testing.T) {
+	var tmpFile *os.File
+	var err error
+	if tmpFile, err = ioutil.TempFile("", "camel-k-"); err != nil {
+		t.Error(err)
+	}
+
+	assert.Nil(t, tmpFile.Close())
+	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestSrcContent), 0o400))
+	fileName := filepath.Base(tmpFile.Name())
+
+	buildCmdOptions, runCmd, _ := initializeRunCmdOptionsWithOutput(t)
+	output, err := test.ExecuteCommand(runCmd, cmdRun, tmpFile.Name(), "-o", "yaml")
+	assert.Equal(t, "yaml", buildCmdOptions.OutputFormat)
+
+	assert.Nil(t, err)
+	assert.Equal(t, fmt.Sprintf(`apiVersion: camel.apache.org/v1
+kind: Integration
+metadata:
+  creationTimestamp: null
+  name: %s
+spec:
+  sources:
+  - content: "\nimport org.apache.camel.builder.RouteBuilder;\n\npublic class Sample
+      extends RouteBuilder {\n  @Override\n  public void configure() throws Exception
+      {\n\t  from(\"timer:tick\")\n        .log(\"Hello Camel K!\");\n  }\n}\n"
+    name: %s
+status: {}
+`, fileName, fileName), output)
+}
+
+func TestTrait(t *testing.T) {
+	var tmpFile *os.File
+	var err error
+	if tmpFile, err = ioutil.TempFile("", "camel-k-"); err != nil {
+		t.Error(err)
+	}
+
+	assert.Nil(t, tmpFile.Close())
+	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestSrcContent), 0o400))
+	fileName := filepath.Base(tmpFile.Name())
+
+	buildCmdOptions, runCmd, _ := initializeRunCmdOptionsWithOutput(t)
+	output, err := test.ExecuteCommand(runCmd, cmdRun, tmpFile.Name(), "-o", "yaml", "-t", "mount.configs=configmap:my-cm", "--connect", "my-service-binding")
+	assert.Equal(t, "yaml", buildCmdOptions.OutputFormat)
+
+	assert.Nil(t, err)
+	assert.Equal(t, fmt.Sprintf(`apiVersion: camel.apache.org/v1
+kind: Integration
+metadata:
+  creationTimestamp: null
+  name: %s
+spec:
+  sources:
+  - content: "\nimport org.apache.camel.builder.RouteBuilder;\n\npublic class Sample
+      extends RouteBuilder {\n  @Override\n  public void configure() throws Exception
+      {\n\t  from(\"timer:tick\")\n        .log(\"Hello Camel K!\");\n  }\n}\n"
+    name: %s
+  traits:
+    mount:
+      configuration:
+        configs:
+        - configmap:my-cm
+    service-binding:
+      configuration:
+        services:
+        - my-service-binding
+status: {}
+`, fileName, fileName), output)
+}
+
+func TestMissingTrait(t *testing.T) {
+	var tmpFile *os.File
+	var err error
+	if tmpFile, err = ioutil.TempFile("", "camel-k-"); err != nil {
+		t.Error(err)
+	}
+
+	assert.Nil(t, tmpFile.Close())
+	assert.Nil(t, ioutil.WriteFile(tmpFile.Name(), []byte(TestSrcContent), 0o400))
+
+	buildCmdOptions, runCmd, _ := initializeRunCmdOptionsWithOutput(t)
+	output, err := test.ExecuteCommand(runCmd, cmdRun, tmpFile.Name(), "-o", "yaml", "-t", "bogus.fail=i-must-fail")
+	assert.Equal(t, "yaml", buildCmdOptions.OutputFormat)
+	assert.Equal(t, "Error: bogus.fail=i-must-fail is not a valid trait property\n", output)
+	assert.NotNil(t, err)
+}
