@@ -31,6 +31,7 @@ import (
 
 	v1 "github.com/apache/camel-k/pkg/apis/camel/v1"
 	"github.com/apache/camel-k/pkg/util"
+	"github.com/apache/camel-k/pkg/util/camel"
 	"github.com/apache/camel-k/pkg/util/defaults"
 	"github.com/apache/camel-k/pkg/util/envvar"
 	"github.com/apache/camel-k/pkg/util/kubernetes"
@@ -68,7 +69,7 @@ type containerTrait struct {
 	Expose *bool `property:"expose" json:"expose,omitempty"`
 	// To configure a different port exposed by the container (default `8080`).
 	Port int `property:"port" json:"port,omitempty"`
-	// To configure a different port name for the port exposed by the container (default `http`).
+	// To configure a different port name for the port exposed by the container. It defaults to `http` only when the `expose` parameter is true.
 	PortName string `property:"port-name" json:"portName,omitempty"`
 	// To configure under which service port the container port is to be exposed (default `80`).
 	ServicePort int `property:"service-port" json:"servicePort,omitempty"`
@@ -167,13 +168,13 @@ func isValidPullPolicy(policy corev1.PullPolicy) bool {
 }
 
 func (t *containerTrait) Apply(e *Environment) error {
-	if e.IntegrationInPhase(v1.IntegrationPhaseInitialization) {
-		return t.configureImageIntegrationKit(e)
+	if err := t.configureImageIntegrationKit(e); err != nil {
+		return err
 	}
 	return t.configureContainer(e)
 }
 
-// IsPlatformTrait overrides base class method
+// IsPlatformTrait overrides base class method.
 func (t *containerTrait) IsPlatformTrait() bool {
 	return true
 }
@@ -185,12 +186,6 @@ func (t *containerTrait) configureImageIntegrationKit(e *Environment) error {
 				"unsupported configuration: a container image has been set in conjunction with an IntegrationKit %v",
 				e.Integration.Spec.IntegrationKit)
 		}
-		// nolint: staticcheck
-		if e.Integration.Spec.Kit != "" {
-			return fmt.Errorf(
-				"unsupported configuration: a container image has been set in conjunction with an IntegrationKit %s",
-				e.Integration.Spec.Kit)
-		}
 
 		kitName := fmt.Sprintf("kit-%s", e.Integration.Name)
 		kit := v1.NewIntegrationKit(e.Integration.Namespace, kitName)
@@ -245,36 +240,33 @@ func (t *containerTrait) configureContainer(e *Environment) error {
 	}
 
 	envvar.SetVal(&container.Env, "CAMEL_K_DIGEST", e.Integration.Status.Digest)
-	envvar.SetVal(&container.Env, "CAMEL_K_CONF", path.Join(basePath, "application.properties"))
-	envvar.SetVal(&container.Env, "CAMEL_K_CONF_D", confDPath)
+	envvar.SetVal(&container.Env, "CAMEL_K_CONF", path.Join(camel.BasePath, "application.properties"))
+	envvar.SetVal(&container.Env, "CAMEL_K_CONF_D", camel.ConfDPath)
 
 	e.addSourcesProperties()
+	if props, err := e.computeApplicationProperties(); err != nil {
+		return err
+	} else if props != nil {
+		e.Resources.Add(props)
+	}
 
 	t.configureResources(e, &container)
-
 	if IsTrue(t.Expose) {
 		t.configureService(e, &container)
 	}
 	t.configureCapabilities(e)
 
+	var containers *[]corev1.Container
+	visited := false
+
 	// Deployment
 	if err := e.Resources.VisitDeploymentE(func(deployment *appsv1.Deployment) error {
 		for _, envVar := range e.EnvVars {
 			envvar.SetVar(&container.Env, envVar)
 		}
-		if props, err := e.computeApplicationProperties(); err != nil {
-			return err
-		} else if props != nil {
-			e.Resources.Add(props)
-		}
-
-		e.configureVolumesAndMounts(
-			&deployment.Spec.Template.Spec.Volumes,
-			&container.VolumeMounts,
-		)
-
-		deployment.Spec.Template.Spec.Containers = append(deployment.Spec.Template.Spec.Containers, container)
 
+		containers = &deployment.Spec.Template.Spec.Containers
+		visited = true
 		return nil
 	}); err != nil {
 		return err
@@ -296,19 +288,9 @@ func (t *containerTrait) configureContainer(e *Environment) error {
 				envvar.SetVar(&container.Env, env)
 			}
 		}
-		if props, err := e.computeApplicationProperties(); err != nil {
-			return err
-		} else if props != nil {
-			e.Resources.Add(props)
-		}
-
-		e.configureVolumesAndMounts(
-			&service.Spec.ConfigurationSpec.Template.Spec.Volumes,
-			&container.VolumeMounts,
-		)
-
-		service.Spec.ConfigurationSpec.Template.Spec.Containers = append(service.Spec.ConfigurationSpec.Template.Spec.Containers, container)
 
+		containers = &service.Spec.ConfigurationSpec.Template.Spec.Containers
+		visited = true
 		return nil
 	}); err != nil {
 		return err
@@ -319,24 +301,18 @@ func (t *containerTrait) configureContainer(e *Environment) error {
 		for _, envVar := range e.EnvVars {
 			envvar.SetVar(&container.Env, envVar)
 		}
-		if props, err := e.computeApplicationProperties(); err != nil {
-			return err
-		} else if props != nil {
-			e.Resources.Add(props)
-		}
-
-		e.configureVolumesAndMounts(
-			&cron.Spec.JobTemplate.Spec.Template.Spec.Volumes,
-			&container.VolumeMounts,
-		)
-
-		cron.Spec.JobTemplate.Spec.Template.Spec.Containers = append(cron.Spec.JobTemplate.Spec.Template.Spec.Containers, container)
 
+		containers = &cron.Spec.JobTemplate.Spec.Template.Spec.Containers
+		visited = true
 		return nil
 	}); err != nil {
 		return err
 	}
 
+	if visited {
+		*containers = append(*containers, container)
+	}
+
 	return nil
 }
 
