@@ -32,8 +32,9 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime/schema"
 
-	"sigs.k8s.io/controller-runtime/pkg/client"
+	ctrl "sigs.k8s.io/controller-runtime/pkg/client"
 
 	v1 "github.com/apache/camel-k/pkg/apis/camel/v1"
 	"github.com/apache/camel-k/pkg/util"
@@ -47,9 +48,11 @@ import (
 
 // The OpenAPI DSL trait is internally used to allow creating integrations from a OpenAPI specs.
 //
-// +camel-k:trait=openapi
+// +camel-k:trait=openapi.
 type openAPITrait struct {
 	BaseTrait `property:",squash"`
+	// The configmaps holding the spec of the OpenAPI
+	Configmaps []string `property:"configmaps" json:"configmaps,omitempty"`
 }
 
 func newOpenAPITrait() Trait {
@@ -58,7 +61,7 @@ func newOpenAPITrait() Trait {
 	}
 }
 
-// IsPlatformTrait overrides base class method
+// IsPlatformTrait overrides base class method.
 func (t *openAPITrait) IsPlatformTrait() bool {
 	return true
 }
@@ -83,6 +86,10 @@ func (t *openAPITrait) Configure(e *Environment) (bool, error) {
 		}
 	}
 
+	if t.Configmaps != nil {
+		return e.IntegrationInPhase(v1.IntegrationPhaseInitialization), nil
+	}
+
 	return false, nil
 }
 
@@ -94,27 +101,71 @@ func (t *openAPITrait) Apply(e *Environment) error {
 	if err != nil {
 		return err
 	}
-
 	defer os.RemoveAll(tmpDir)
 
-	for i, resource := range e.Integration.Spec.Resources {
+	generatedFromResources, err := t.generateFromResources(e, tmpDir)
+	if err != nil {
+		return err
+	}
+	generatedFromConfigmaps, err := t.generateFromConfigmaps(e, tmpDir)
+	if err != nil {
+		return err
+	}
+	if len(generatedFromConfigmaps) > 0 {
+		generatedFromResources = append(generatedFromResources, generatedFromConfigmaps...)
+	}
+	e.Integration.Status.GeneratedSources = generatedFromResources
+
+	return nil
+}
+
+func (t *openAPITrait) generateFromResources(e *Environment, tmpDir string) ([]v1.SourceSpec, error) {
+	dataSpecs := make([]v1.DataSpec, 0, len(e.Integration.Spec.Resources))
+	for _, resource := range e.Integration.Spec.Resources {
 		if resource.Type != v1.ResourceTypeOpenAPI {
 			continue
 		}
 		if resource.Name == "" {
-			return fmt.Errorf("no name defined for the openapi resource: %v", resource)
+			return nil, fmt.Errorf("no name defined for the openapi resource: %v", resource)
 		}
+		dataSpecs = append(dataSpecs, resource.DataSpec)
+	}
 
+	return t.generateFromDataSpecs(e, tmpDir, dataSpecs)
+}
+
+func (t *openAPITrait) generateFromConfigmaps(e *Environment, tmpDir string) ([]v1.SourceSpec, error) {
+	dataSpecs := make([]v1.DataSpec, 0, len(t.Configmaps))
+	for _, configmap := range t.Configmaps {
+		// verify if it was autogenerated
+		cm, err := kubernetes.GetUnstructured(e.Ctx, e.Client, schema.GroupVersionKind{Group: "", Version: "v1", Kind: "ConfigMap"},
+			configmap, e.Integration.Namespace)
+		if err == nil && cm != nil && cm.GetLabels()[kubernetes.ConfigMapAutogenLabel] == "true" {
+			refCm := kubernetes.NewConfigMap(e.Integration.Namespace, configmap, "", "", "", nil)
+			e.Resources.Add(refCm)
+		}
+		// Iterate over each configmap key which may hold a different OpenAPI spec
+		for k, v := range cm.UnstructuredContent()["data"].(map[string]interface{}) {
+			dataSpecs = append(dataSpecs, v1.DataSpec{
+				Name:        k,
+				Content:     v.(string),
+				Compression: false,
+			})
+		}
+	}
+
+	return t.generateFromDataSpecs(e, tmpDir, dataSpecs)
+}
+
+func (t *openAPITrait) generateFromDataSpecs(e *Environment, tmpDir string, specs []v1.DataSpec) ([]v1.SourceSpec, error) {
+	generatedSources := make([]v1.SourceSpec, 0, len(e.Integration.Status.GeneratedSources))
+	for i, resource := range specs {
 		generatedContentName := fmt.Sprintf("%s-openapi-%03d", e.Integration.Name, i)
-
+		generatedSourceName := strings.TrimSuffix(resource.Name, filepath.Ext(resource.Name)) + ".xml"
 		// Generate configmap or reuse existing one
 		if err := t.generateOpenAPIConfigMap(e, resource, tmpDir, generatedContentName); err != nil {
-			return errors.Wrapf(err, "cannot generate configmap for openapi resource %s", resource.Name)
+			return nil, errors.Wrapf(err, "cannot generate configmap for openapi resource %s", resource.Name)
 		}
-
-		generatedSourceName := strings.TrimSuffix(resource.Name, filepath.Ext(resource.Name)) + ".xml"
-		generatedSources := make([]v1.SourceSpec, 0, len(e.Integration.Status.GeneratedSources))
-
 		if e.Integration.Status.GeneratedSources != nil {
 			// Filter out the previously generated source
 			for _, x := range e.Integration.Status.GeneratedSources {
@@ -124,7 +175,7 @@ func (t *openAPITrait) Apply(e *Environment) error {
 			}
 		}
 
-		// Add an additional source that references the config map
+		// Add a source that references the config map
 		generatedSources = append(generatedSources, v1.SourceSpec{
 			DataSpec: v1.DataSpec{
 				Name:        generatedSourceName,
@@ -133,16 +184,14 @@ func (t *openAPITrait) Apply(e *Environment) error {
 			},
 			Language: v1.LanguageXML,
 		})
-
-		e.Integration.Status.GeneratedSources = generatedSources
 	}
 
-	return nil
+	return generatedSources, nil
 }
 
-func (t *openAPITrait) generateOpenAPIConfigMap(e *Environment, resource v1.ResourceSpec, tmpDir, generatedContentName string) error {
+func (t *openAPITrait) generateOpenAPIConfigMap(e *Environment, resource v1.DataSpec, tmpDir, generatedContentName string) error {
 	cm := corev1.ConfigMap{}
-	key := client.ObjectKey{
+	key := ctrl.ObjectKey{
 		Namespace: e.Integration.Namespace,
 		Name:      generatedContentName,
 	}
@@ -174,7 +223,7 @@ func (t *openAPITrait) generateOpenAPIConfigMap(e *Environment, resource v1.Reso
 	return t.createNewOpenAPIConfigMap(e, resource, tmpDir, generatedContentName)
 }
 
-func (t *openAPITrait) createNewOpenAPIConfigMap(e *Environment, resource v1.ResourceSpec, tmpDir, generatedContentName string) error {
+func (t *openAPITrait) createNewOpenAPIConfigMap(e *Environment, resource v1.DataSpec, tmpDir, generatedContentName string) error {
 	tmpDir = path.Join(tmpDir, generatedContentName)
 	err := os.MkdirAll(tmpDir, os.ModePerm)
 	if err != nil {
@@ -192,7 +241,7 @@ func (t *openAPITrait) createNewOpenAPIConfigMap(e *Environment, resource v1.Res
 	in := path.Join(tmpDir, resource.Name)
 	out := path.Join(tmpDir, "openapi-dsl.xml")
 
-	err = ioutil.WriteFile(in, content, 0o644)
+	err = ioutil.WriteFile(in, content, 0o400)
 	if err != nil {
 		return err
 	}
@@ -204,25 +253,36 @@ func (t *openAPITrait) createNewOpenAPIConfigMap(e *Environment, resource v1.Res
 
 	mc := maven.NewContext(tmpDir)
 	mc.LocalRepository = e.Platform.Status.Build.Maven.LocalRepository
+	mc.AdditionalArguments = e.Platform.Status.Build.Maven.CLIOptions
 	mc.AddArgument("-Dopenapi.spec=" + in)
 	mc.AddArgument("-Ddsl.out=" + out)
 
-	settings, err := kubernetes.ResolveValueSource(e.Ctx, e.Client, e.Platform.Namespace, &e.Platform.Status.Build.Maven.Settings)
+	if settings, err := kubernetes.ResolveValueSource(e.Ctx, e.Client, e.Platform.Namespace, &e.Platform.Status.Build.Maven.Settings); err != nil {
+		return err
+	} else if settings != "" {
+		mc.UserSettings = []byte(settings)
+	}
+
+	settings, err := maven.NewSettings(maven.DefaultRepositories, maven.ProxyFromEnvironment)
 	if err != nil {
 		return err
 	}
-	if settings != "" {
-		mc.SettingsContent = []byte(settings)
+	data, err := settings.MarshalBytes()
+	if err != nil {
+		return err
 	}
+	mc.GlobalSettings = data
+	// nolint: staticcheck
+	secrets := mergeSecrets(e.Platform.Status.Build.Maven.CASecrets, e.Platform.Status.Build.Maven.CASecret)
 
-	if e.Platform.Status.Build.Maven.CASecret != nil {
-		certData, err := kubernetes.GetSecretRefData(e.Ctx, e.Client, e.Platform.Namespace, e.Platform.Status.Build.Maven.CASecret)
+	if secrets != nil {
+		certsData, err := kubernetes.GetSecretsRefData(e.Ctx, e.Client, e.Platform.Namespace, secrets)
 		if err != nil {
 			return err
 		}
 		trustStoreName := "trust.jks"
 		trustStorePass := jvm.NewKeystorePassword()
-		err = jvm.GenerateKeystore(e.Ctx, tmpDir, trustStoreName, trustStorePass, certData)
+		err = jvm.GenerateKeystore(e.Ctx, tmpDir, trustStoreName, trustStorePass, certsData)
 		if err != nil {
 			return err
 		}
@@ -239,7 +299,7 @@ func (t *openAPITrait) createNewOpenAPIConfigMap(e *Environment, resource v1.Res
 		return err
 	}
 
-	content, err = ioutil.ReadFile(out)
+	content, err = util.ReadFile(out)
 	if err != nil {
 		return err
 	}
@@ -290,6 +350,16 @@ func (t *openAPITrait) createNewOpenAPIConfigMap(e *Environment, resource v1.Res
 	return nil
 }
 
+func mergeSecrets(secrets []corev1.SecretKeySelector, secret *corev1.SecretKeySelector) []corev1.SecretKeySelector {
+	if secrets == nil && secret == nil {
+		return nil
+	}
+	if secret == nil {
+		return secrets
+	}
+	return append(secrets, *secret)
+}
+
 func (t *openAPITrait) generateMavenProject(e *Environment) (maven.Project, error) {
 	if e.CamelCatalog == nil {
 		return maven.Project{}, errors.New("unknown camel catalog")
