@@ -20,6 +20,7 @@ package trait
 import (
 	appsv1 "k8s.io/api/apps/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+
 	serving "knative.dev/serving/pkg/apis/serving/v1"
 
 	v1 "github.com/apache/camel-k/pkg/apis/camel/v1"
@@ -28,7 +29,7 @@ import (
 // The Owner trait ensures that all created resources belong to the integration being created
 // and transfers annotations and labels on the integration onto these owned resources.
 //
-// +camel-k:trait=owner
+// +camel-k:trait=owner.
 type ownerTrait struct {
 	BaseTrait `property:",squash"`
 	// The set of annotations to be transferred
@@ -52,7 +53,7 @@ func (t *ownerTrait) Configure(e *Environment) (bool, error) {
 		return false, nil
 	}
 
-	return e.IntegrationInPhase(v1.IntegrationPhaseInitialization, v1.IntegrationPhaseDeploying, v1.IntegrationPhaseRunning), nil
+	return e.IntegrationInPhase(v1.IntegrationPhaseInitialization) || e.IntegrationInRunningPhases(), nil
 }
 
 func (t *ownerTrait) Apply(e *Environment) error {
@@ -110,7 +111,7 @@ func (t *ownerTrait) Apply(e *Environment) error {
 	return nil
 }
 
-// IsPlatformTrait overrides base class method
+// IsPlatformTrait overrides base class method.
 func (t *ownerTrait) IsPlatformTrait() bool {
 	return true
 }
